@@ -9,10 +9,10 @@ use constant DEBUG      => $ENV{SWAGGER2_DEBUG};
 use constant IO_LOGGING => $ENV{SWAGGER2_IO_LOGGING};    # EXPERIMENTAL
 
 my $SKIP_OP_RE = qr(By|From|For|In|Of|To|With);
+my $LAYOUT = Mojo::Loader::data_section(__PACKAGE__, 'layouts/mojolicious_plugin_swagger.html.ep');
 
-has url             => '';
-has _validator      => sub { Swagger2::SchemaValidator->new; };
-has _json_validator => sub { JSON::Validator->new; };
+has url => '';
+has _validator => sub { Swagger2::SchemaValidator->new; };
 
 sub dispatch_to_swagger {
   return undef unless $_[1]->{op} and $_[1]->{id} and ref $_[1]->{params} eq 'HASH';
@@ -30,7 +30,7 @@ sub dispatch_to_swagger {
   for my $p (@{$defaults->{swagger_operation_spec}{parameters} || []}) {
     my $name  = $p->{name};
     my $value = $data->{params}{$name} // $p->{default};
-    my @e     = $self->_validate_input_value($p, $name => $value);
+    my @e     = $self->_validator->_validate_request_value($p, $name => $value);
     $input->{$name} = $value unless @e;
     push @errors, @e;
   }
@@ -48,8 +48,8 @@ sub dispatch_to_swagger {
       my $delay  = shift;
       my $data   = shift;
       my $status = shift || 200;
-      my @errors
-        = $self->_validate_response($c, $data, $defaults->{swagger_operation_spec}, $status);
+      my @errors = $self->_validator->validate_response($c, $defaults->{swagger_operation_spec},
+        $status, $data);
 
       return $c->$reply($data, $status) unless @errors;
       warn "[Swagger2] Invalid response: @errors\n" if DEBUG;
@@ -73,11 +73,18 @@ sub register {
   $swagger = $swagger->expand;
   $paths   = $swagger->api_spec->get('/paths') || {};
 
+  $app->plugin(PODRenderer => {no_perldoc => 1}) unless $app->renderer->helpers->{pod_to_html};
+
   if ($config->{validate} // 1) {
     my @errors = $swagger->validate;
     die join "\n", "Swagger2: Invalid spec:", @errors if @errors;
   }
-  if (!$app->plugins->has_subscribers('swagger_route_added')) {
+
+  if ($app->plugins->has_subscribers('swagger_route_added')) {
+    warn
+      "swagger_route_added hook will be deprecated. https://github.com/jhthorsen/swagger2/issues/65";
+  }
+  else {
     $app->hook(swagger_route_added => \&_on_route_added);
   }
 
@@ -170,26 +177,26 @@ sub _generate_request_handler {
 
   my $handler = sub {
     my $c = shift;
-    my ($e, $v, $input);
+    my ($e, @errors, %input);
 
     return $c->render_swagger(_error($e), {}, 501) if $e = _find_action($c, $defaults);
     $c = $defaults->{controller}->new(%$c);
-    ($v, $input) = $self->_validate_input($c, $op_spec);
+    @errors = $self->_validator->validate_request($c, $op_spec, \%input);
 
-    _io_error($c, Input => $v->{errors}) if IO_LOGGING and @{$v->{errors}};
-    return $c->render_swagger($v, {}, 400) if @{$v->{errors}};
+    _io_error($c, Input => \@errors) if IO_LOGGING and @errors;
+    return $c->render_swagger({errors => \@errors}, {}, 400) if @errors;
     return $c->delay(
       sub {
         my $action = $defaults->{action};
         $c->app->log->debug(
           qq(Swagger2 routing to controller "$defaults->{controller}" and action "$action"));
-        $c->$action($input, shift->begin);
+        $c->$action(\%input, shift->begin);
       },
       sub {
         my $delay  = shift;
         my $data   = shift;
         my $status = shift || 200;
-        my @errors = $self->_validate_response($c, $data, $op_spec, $status);
+        my @errors = $self->_validator->validate_response($c, $op_spec, $status, $data);
 
         return $c->render_swagger({}, $data, $status) unless @errors;
         _io_error($c, Output => \@errors) if IO_LOGGING and @errors;
@@ -231,142 +238,28 @@ sub _on_route_added {
 
 sub _render_spec {
   my ($c, $swagger) = @_;
-  my $spec = $swagger->api_spec->data;
+  my $format = $c->stash('format') || 'json';
+  my $spec   = $swagger->api_spec->data;
+  my $url    = $c->req->url->to_abs;
 
   local $spec->{id};
   delete $spec->{id};
-  local $spec->{host} = $c->req->url->to_abs->host_port;
-  $c->render(json => $spec);
-}
+  local $spec->{host} = $url->host_port;
+  $swagger->base_url->host($url->host)->port($url->port);
 
-sub _validate_input {
-  my ($self, $c, $op_spec) = @_;
-  my (%cache, %input, @errors);
-
-  for my $p (@{$op_spec->{parameters} || []}) {
-    my ($in, $name, $type) = @$p{qw(in name type)};
-    my ($exists, $value);
-
-    if ($in eq 'body') {
-      $value  = $c->req->json;
-      $exists = $c->req->body_size;
-    }
-    else {
-      $value = $cache{$in} ||= do {
-            $in eq 'query'    ? $c->req->url->query->to_hash
-          : $in eq 'path'     ? $c->match->stack->[-1]
-          : $in eq 'formData' ? $c->req->body_params->to_hash
-          : $in eq 'header'   ? $c->req->headers->to_hash
-          :                     {};
-      };
-      $exists = exists $value->{$name};
-      $value  = $value->{$name};
-    }
-
-    if (ref $p->{items} eq 'HASH' and $p->{collectionFormat}) {
-      $value = _coerce_by_collection_format($value, $p);
-    }
-
-    if ($type and defined($value //= $p->{default})) {
-      if (($type eq 'integer' or $type eq 'number') and $value =~ /^-?\d/) {
-        $value += 0;
-      }
-      elsif ($type eq 'boolean') {
-        $value = (!$value or $value eq 'false') ? Mojo::JSON->false : Mojo::JSON->true;
-      }
-    }
-
-    my @e = $self->_validate_input_value($p, $name => $value);
-    $input{$name} = $value if !@e and ($exists or exists $p->{default});
-    push @errors, @e;
+  if ($format eq 'text') {
+    $c->render(text => $swagger->pod->to_string);
   }
-
-  return {errors => \@errors}, \%input;
-}
-
-sub _validate_input_value {
-  my ($self, $p, $name, $value) = @_;
-  my $type = $p->{type} || 'object';
-  my @e;
-
-  return if !defined $value and !Swagger2::_is_true($p->{required});
-
-  my $schema = {
-    properties => {$name => $p->{'x-json-schema'} || $p->{schema} || $p},
-    required => [$p->{required} ? ($name) : ()]
-  };
-  my $in = $p->{in};
-
-  if ($in eq 'body') {
-    warn "[Swagger2] Validate $in $name\n" if DEBUG;
-    if ($p->{'x-json-schema'}) {
-      return $self->_json_validator->validate({$name => $value}, $schema);
-    }
-    else {
-      return $self->_validator->validate_input({$name => $value}, $schema);
-    }
-  }
-  elsif (defined $value) {
-    warn "[Swagger2] Validate $in $name=$value\n" if DEBUG;
-    return $self->_validator->validate_input({$name => $value}, $schema);
+  elsif ($format eq 'html') {
+    $c->render(
+      handler => 'ep',
+      inline  => $LAYOUT,
+      pod     => $c->pod_to_html($swagger->pod->to_string)
+    );
   }
   else {
-    warn "[Swagger2] Validate $in $name=undef\n" if DEBUG;
-    return $self->_validator->validate_input({$name => $value}, $schema);
+    $c->render(json => $spec);
   }
-
-  return;
-}
-
-sub _validate_response {
-  my ($self, $c, $data, $op_spec, $status) = @_;
-  my $headers = $c->res->headers;
-  my @errors;
-
-  if (my $blueprint = $op_spec->{responses}{$status} || $op_spec->{responses}{default}) {
-    my $input = $headers->to_hash(1);
-
-    for my $n (keys %{$blueprint->{headers} || {}}) {
-      my $p = $blueprint->{headers}{$n};
-
-      # jhthorsen: I think that the only way to make a header required,
-      # is by defining "array" and "minItems" >= 1.
-      if ($p->{type} eq 'array') {
-        push @errors, $self->_validator->validate($input->{$n}, $p);
-      }
-      elsif ($input->{$n}) {
-        push @errors, $self->_validator->validate($input->{$n}[0], $p);
-        $headers->header($n => $input->{$n}[0] ? 'true' : 'false')
-          if $p->{type} eq 'boolean' and !@errors;
-      }
-    }
-
-    if ($blueprint->{'x-json-schema'}) {
-      warn "[Swagger2] Validate using x-json-schema\n" if DEBUG;
-      push @errors, $self->_json_validator->validate($data, $blueprint->{'x-json-schema'});
-    }
-    elsif ($blueprint->{schema}) {
-      warn "[Swagger2] Validate using schema\n" if DEBUG;
-      push @errors, $self->_validator->validate($data, $blueprint->{schema});
-    }
-  }
-  else {
-    push @errors, $self->_validator->validate($data, {});
-  }
-
-  return @errors;
-}
-
-sub _coerce_by_collection_format {
-  my ($value, $schema) = @_;
-  my $re = $Swagger2::SchemaValidator::COLLECTION_RE{$schema->{collectionFormat}} || '';
-  my $type = $schema->{items}{type} || '';
-  my @data;
-
-  return [ref $value ? @$value : $value] unless $re;
-  defined and push @data, split /$re/ for ref $value ? @$value : $value;
-  return [map { $_ + 0 } @data] if $type eq 'integer' or $type eq 'number';
-  return \@data;
 }
 
 sub _error {
@@ -506,15 +399,7 @@ or look at L<Swagger2::Guides::Tutorial> for an introduction.
 
 =head2 swagger_route_added
 
-  $app->hook(swagger_route_added => sub {
-    my ($app, $r) = @_;
-    my $op_spec = $r->pattern->defaults->{swagger_operation_spec};
-    # ...
-  });
-
-The "swagger_route_added" event will be emitted on the application object
-for every route that is added by this plugin. This can be useful if you
-want to do things like specifying a custom route name.
+This hook will be DEPRECATED! See L<https://github.com/jhthorsen/swagger2/issues/65>.
 
 =head1 STASH VARIABLES
 
@@ -647,3 +532,33 @@ the terms of the Artistic License version 2.0.
 Jan Henning Thorsen - C<jhthorsen@cpan.org>
 
 =cut
+
+__DATA__
+@@ layouts/mojolicious_plugin_swagger.html.ep
+<!DOCTYPE html>
+<html>
+  <head>
+    <title><%= $swagger->api_spec->get('/info/title') %></title>
+    <style>
+      body {
+        background: #fefefe;
+        font-family: sans-serif;
+        font-size: 16px;
+      }
+      pre {
+        background: #f4f4f4;
+        border: 1px solid #ddd;
+        padding: 5px;
+      }
+      .container {
+        max-width: 60em;
+        margin: 3em auto;
+      }
+    </style>
+  </head>
+  <body>
+    <div class="container">
+      <%= $pod %>
+    </div>
+  </body>
+</html>
